@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2018 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -76,7 +76,7 @@ class TabbedBrowser(QWidget):
         _local_marks: Jump markers local to each page
         _global_marks: Jump markers used across all pages
         default_window_icon: The qutebrowser window icon
-        private: Whether private browsing is on for this window.
+        is_private: Whether private browsing is on for this window.
 
     Signals:
         cur_progress: Progress of the current tab changed (load_progress).
@@ -102,7 +102,7 @@ class TabbedBrowser(QWidget):
     cur_url_changed = pyqtSignal(QUrl)
     cur_link_hovered = pyqtSignal(str)
     cur_scroll_perc_changed = pyqtSignal(int, int)
-    cur_load_status_changed = pyqtSignal(str)
+    cur_load_status_changed = pyqtSignal(usertypes.LoadStatus)
     cur_fullscreen_requested = pyqtSignal(bool)
     cur_caret_selection_toggled = pyqtSignal(bool)
     close_window = pyqtSignal()
@@ -131,7 +131,7 @@ class TabbedBrowser(QWidget):
         self._local_marks = {}
         self._global_marks = {}
         self.default_window_icon = self.widget.window().windowIcon()
-        self.private = private
+        self.is_private = private
         config.instance.changed.connect(self._on_config_changed)
 
     def __repr__(self):
@@ -222,6 +222,7 @@ class TabbedBrowser(QWidget):
             self._filter.create(self.cur_caret_selection_toggled, tab))
         # misc
         tab.scroller.perc_changed.connect(self.on_scroll_pos_changed)
+        tab.scroller.before_jump_requested.connect(lambda: self.set_mark("'"))
         tab.url_changed.connect(
             functools.partial(self.on_url_changed, tab))
         tab.title_changed.connect(
@@ -243,9 +244,9 @@ class TabbedBrowser(QWidget):
         tab.audio.recently_audible_changed.connect(
             functools.partial(self._on_audio_changed, tab))
         tab.new_tab_requested.connect(self.tabopen)
-        if not self.private:
+        if not self.is_private:
             web_history = objreg.get('web-history')
-            tab.add_history_item.connect(web_history.add_from_tab)
+            tab.history_item_triggered.connect(web_history.add_from_tab)
 
     def current_url(self):
         """Get the URL of the current tab.
@@ -303,12 +304,12 @@ class TabbedBrowser(QWidget):
             if last_close == 'close':
                 self.close_window.emit()
             elif last_close == 'blank':
-                self.openurl(QUrl('about:blank'), newtab=True)
+                self.load_url(QUrl('about:blank'), newtab=True)
             elif last_close == 'startpage':
                 for url in config.val.url.start_pages:
-                    self.openurl(url, newtab=True)
+                    self.load_url(url, newtab=True)
             elif last_close == 'default-page':
-                self.openurl(config.val.url.default_page, newtab=True)
+                self.load_url(config.val.url.default_page, newtab=True)
 
     def _remove_tab(self, tab, *, add_undo=True, new_undo=True, crashed=False):
         """Remove a tab from the tab list and delete it properly.
@@ -345,7 +346,7 @@ class TabbedBrowser(QWidget):
             urlutils.invalid_url_error(tab.url(), "saving tab")
         elif add_undo:
             try:
-                history_data = tab.history.serialize()
+                history_data = tab.history.private_api.serialize()
             except browsertab.WebTabError:
                 pass  # special URL
             else:
@@ -356,7 +357,7 @@ class TabbedBrowser(QWidget):
                 else:
                     self._undo_stack[-1].append(entry)
 
-        tab.shutdown()
+        tab.private_api.shutdown()
         self.widget.removeTab(idx)
         if not crashed:
             # WORKAROUND for a segfault when we delete the crashed tab.
@@ -391,11 +392,11 @@ class TabbedBrowser(QWidget):
             else:
                 newtab = self.tabopen(background=False, idx=entry.index)
 
-            newtab.history.deserialize(entry.history)
+            newtab.history.private_api.deserialize(entry.history)
             self.widget.set_tab_pinned(newtab, entry.pinned)
 
     @pyqtSlot('QUrl', bool)
-    def openurl(self, url, newtab):
+    def load_url(self, url, newtab):
         """Open a URL, used as a slot.
 
         Args:
@@ -406,7 +407,7 @@ class TabbedBrowser(QWidget):
         if newtab or self.widget.currentWidget() is None:
             self.tabopen(url, background=False)
         else:
-            self.widget.currentWidget().openurl(url)
+            self.widget.currentWidget().load_url(url)
 
     @pyqtSlot(int)
     def on_tab_close_requested(self, idx):
@@ -466,14 +467,15 @@ class TabbedBrowser(QWidget):
 
         if (config.val.tabs.tabs_are_windows and self.widget.count() > 0 and
                 not ignore_tabs_are_windows):
-            window = mainwindow.MainWindow(private=self.private)
+            window = mainwindow.MainWindow(private=self.is_private)
             window.show()
             tabbed_browser = objreg.get('tabbed-browser', scope='window',
                                         window=window.win_id)
             return tabbed_browser.tabopen(url=url, background=background,
                                           related=related)
 
-        tab = browsertab.create(win_id=self._win_id, private=self.private,
+        tab = browsertab.create(win_id=self._win_id,
+                                private=self.is_private,
                                 parent=self.widget)
         self._connect_tab_signals(tab)
 
@@ -482,7 +484,7 @@ class TabbedBrowser(QWidget):
         self.widget.insertTab(idx, tab, "")
 
         if url is not None:
-            tab.openurl(url)
+            tab.load_url(url)
 
         if background is None:
             background = config.val.tabs.background
@@ -527,14 +529,20 @@ class TabbedBrowser(QWidget):
         else:
             pos = config.val.tabs.new_position.unrelated
         if pos == 'prev':
-            idx = self._tab_insert_idx_left
-            # On first sight, we'd think we have to decrement
-            # self._tab_insert_idx_left here, as we want the next tab to be
-            # *before* the one we just opened. However, since we opened a tab
-            # *before* the currently focused tab, indices will shift by
-            # 1 automatically.
+            if config.val.tabs.new_position.stacking:
+                idx = self._tab_insert_idx_left
+                # On first sight, we'd think we have to decrement
+                # self._tab_insert_idx_left here, as we want the next tab to be
+                # *before* the one we just opened. However, since we opened a
+                # tab *before* the currently focused tab, indices will shift by
+                # 1 automatically.
+            else:
+                idx = self.widget.currentIndex()
         elif pos == 'next':
-            idx = self._tab_insert_idx_right
+            if config.val.tabs.new_position.stacking:
+                idx = self._tab_insert_idx_right
+            else:
+                idx = self.widget.currentIndex() + 1
             self._tab_insert_idx_right += 1
         elif pos == 'first':
             idx = 0
@@ -578,10 +586,20 @@ class TabbedBrowser(QWidget):
     @pyqtSlot()
     def on_cur_load_started(self):
         """Leave insert/hint mode when loading started."""
-        modeman.leave(self._win_id, usertypes.KeyMode.insert, 'load started',
-                      maybe=True)
-        modeman.leave(self._win_id, usertypes.KeyMode.hint, 'load started',
-                      maybe=True)
+        try:
+            url = self.current_url()
+            if not url.isValid():
+                url = None
+        except qtutils.QtValueError:
+            url = None
+        if config.instance.get('input.insert_mode.leave_on_load',
+                               url=url):
+            modeman.leave(self._win_id, usertypes.KeyMode.insert,
+                          'load started', maybe=True)
+        else:
+            log.modes.debug("Ignoring leave_on_load request due to setting.")
+        modeman.leave(self._win_id, usertypes.KeyMode.hint,
+                      'load started', maybe=True)
 
     @pyqtSlot(browsertab.AbstractTab, str)
     def on_title_changed(self, tab, text):
@@ -746,7 +764,7 @@ class TabbedBrowser(QWidget):
         self.widget.update_tab_title(idx)
         if idx == self.widget.currentIndex():
             self._update_window_title()
-            tab.handle_auto_insert_mode(ok)
+            tab.private_api.handle_auto_insert_mode(ok)
 
     @pyqtSlot()
     def on_scroll_pos_changed(self):
@@ -872,7 +890,7 @@ class TabbedBrowser(QWidget):
                         self.cur_load_finished.disconnect(callback)
                         tab.scroller.to_point(point)
 
-                self.openurl(url, newtab=False)
+                self.load_url(url, newtab=False)
                 self.cur_load_finished.connect(callback)
             else:
                 message.error("Mark {} is not set".format(key))
@@ -884,8 +902,7 @@ class TabbedBrowser(QWidget):
             # save the pre-jump position in the special ' mark
             # this has to happen after we read the mark, otherwise jump_mark
             # "'" would just jump to the current position every time
-            self.set_mark("'")
-
+            tab.scroller.before_jump_requested.emit()
             tab.scroller.to_point(point)
         else:
             message.error("Mark {} is not set".format(key))
